package com.szyd.uhfdemo;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;

import com.szyd.event.BackResult;
import com.szyd.event.GetRFIDThread;
import com.szyd.util.DataAdapter;
import com.szyd.util.MLog;
import com.szyd.util.MUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LeftFragment extends Fragment implements View.OnClickListener, BackResult, AdapterView.OnItemSelectedListener {

    private TextView read_RFID;
    private TextView tagNumbers, readNumbers, useTimes, epcShow, tidShow;
    private Spinner invtDataTypeSet;
    private View epc_tid_show_divd_Line;
    private String tagNumber, readNumber, takeTime;

    private long startTime, usTim, pauseTime;
    private DataAdapter mListAdapter;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        tagNumber = getString(R.string.tag_number) + ":";
        readNumber = getString(R.string.read_number) + ":";
        takeTime = getString(R.string.user_time) + ":";
        return inflater.inflate(R.layout.fragment_left, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
    }

    private void initView(View v) {
        read_RFID = v.findViewById(R.id.read_RFID);
        v.findViewById(R.id.clear_Data).setOnClickListener(this); //清空数据
        ListView specific_Msg = v.findViewById(R.id.specific_Msg);

        tagNumbers = v.findViewById(R.id.tagNumbers);
        readNumbers = v.findViewById(R.id.readNumbers);
        useTimes = v.findViewById(R.id.useTimes);

        //设置盘点数据返回模式
        epcShow = v.findViewById(R.id.epcShow);
        tidShow = v.findViewById(R.id.tidShow);
        epc_tid_show_divd_Line = v.findViewById(R.id.epc_tid_show_divd_Line);
        if (MyApp.getMyApp().getIdataLib().ifJ06()) {//是否为小i设备
            v.findViewById(R.id.invDataSet).setVisibility(View.GONE);
        }
        invtDataTypeSet = v.findViewById(R.id.invModSet);
        invtDataTypeSet.setOnItemSelectedListener(this);
        if (MyApp.currentInvtDataType > -1) {
            refreshTagDataTypeShow(MyApp.currentInvtDataType);
            invtDataTypeSet.setSelection(MyApp.currentInvtDataType);
        }


        read_RFID.setOnClickListener(this);
        read_RFID.setText(GetRFIDThread.getInstance().isIfPostMsg() ? R.string.stop_rfid : R.string.read_rfid);

        mListAdapter = new DataAdapter(getActivity(), realDataMap, realKeyList, tidList);
        specific_Msg.setAdapter(mListAdapter);
        GetRFIDThread.getInstance().setBackResult(this);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.read_RFID:
                startOrStopRFID();
                break;
            case R.id.clear_Data://清除显示数据
                if (GetRFIDThread.getInstance().isIfPostMsg()) {
                    MUtil.show(R.string.notice_clean_data);
                } else {
                    clearData();
                }
                break;
        }
    }

    private void clearData() {
        dataMap.clear();
        realDataMap.clear();
        realKeyList.clear();
        tidList.clear();
        usTim = pauseTime = 0;
        useTimes.setText(takeTime);
        tagNumbers.setText(tagNumber);
        readNumbers.setText(readNumber);
        mListAdapter.notifyDataSetChanged();
    }

    //开启或停止RFID模块
    public void startOrStopRFID() {
        boolean flag = !GetRFIDThread.getInstance().isIfPostMsg();
        invtDataTypeSet.setEnabled(!flag);
        if (flag) {
            MyApp.getMyApp().getIdataLib().startInventoryTag();
            long tmepTime = pauseTime;
            startTime = System.currentTimeMillis() - tmepTime;
        } else {
            MyApp.getMyApp().getIdataLib().stopInventory();
        }
        GetRFIDThread.getInstance().setIfPostMsg(flag);
        read_RFID.setText(flag ? R.string.stop_rfid : R.string.read_rfid);
    }

    //private long rNumber = 0;//读取次数

    private Map<String, Integer> dataMap = new HashMap<>(); //数据

    //扫描到的EPC数据
    @Override
    public void postResult(String[] tagData) {
        //++rNumber;
        String tid = tagData[0];//获取TID
        String epc = tagData[1]; //拿到EPC
        MLog.e("tid = " + tid + " epc = " + epc);

        //showDialog(epc);
        Integer number = dataMap.get(epc);//如果已存在，就拿到数量
        if (number == null) {
            dataMap.put(epc, 1);
            updateUI(epc, tid, 1);
        } else {
            int newNB = number + 1;
            dataMap.put(epc, ++newNB);
            updateUI(epc, tid, newNB);
        }
    }

    @Override
    public void postInventoryRate(final long rate) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                readNumbers.setText(rate + readNumber); // 张数/秒
            }
        });
    }

    @Override
    public void onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_F8) //把枪按钮被按下,默认值为138
            startOrStopRFID();
    }

    //E280116060000208D8363445  ecp    // E28011602000644506C6091B tid
    private Map<String, Integer> realDataMap = new HashMap<>(); //数据
    private List<String> realKeyList = new ArrayList<>(); //epc数据作为key
    private List<String> tidList = new ArrayList<>();//tid

    private void updateUI(final String epc, final String tid, final int readNumberss) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (readNumberss > 1) {
                    realDataMap.put(epc, readNumberss); //修改数量
                } else {
                    realDataMap.put(epc, 1);
                    realKeyList.add(epc);
                    tidList.add(tid);
                }
                long endTime = System.currentTimeMillis();
                pauseTime = usTim = endTime - startTime; //RFID开始到结束的获取时间
                useTimes.setText(takeTime + usTim); //花费的时间
                tagNumbers.setText(tagNumber + realDataMap.size());//标签数量
                mListAdapter.notifyDataSetChanged();
                if (MyApp.ifOpenSound)
                    MyApp.getMyApp().playSound();
            }
        });
    }

    private void refreshTagDataTypeShow(int position) {
        clearData();
        epcShow.setVisibility(position == 1 ? View.GONE : View.VISIBLE);
        tidShow.setVisibility(position == 0 ? View.GONE : View.VISIBLE);
        epc_tid_show_divd_Line.setVisibility(position < 1 ? View.GONE : View.VISIBLE);
    }

    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        MyApp.currentInvtDataType = position;
        mListAdapter.setMode(position);
        refreshTagDataTypeShow(position);
        position = Math.min(position, 1);
        MyApp.getMyApp().getIdataLib().readTagModeSet(position, false);
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {

    }
}
